@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
- * Copyright (C) 1999-2000 Thomas Roessler <roessler@guug.de>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1999 Thomas Roessler <roessler@guug.de>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -357,7 +357,8 @@ int mutt_is_autoview (BODY *b, const char *type)
 }
 
 /* returns -1 on error, 0 or the return code from mutt_do_pager() on success */
-int mutt_view_attachment (FILE *fp, BODY *a, int flag)
+int mutt_view_attachment (FILE *fp, BODY *a, int flag, HEADER *hdr,
+			  ATTACHPTR **idx, short idxlen)
 {
   char tempfile[_POSIX_PATH_MAX] = "";
   char pagerfile[_POSIX_PATH_MAX] = "";
@@ -563,6 +564,10 @@ int mutt_view_attachment (FILE *fp, BODY *a, int flag)
     info.fp = fp;
     info.bdy = a;
     info.ctx = Context;
+    info.idx = idx;
+    info.idxlen = idxlen;
+    info.hdr = hdr;
+
     rc = mutt_do_pager (descrip, pagerfile,
 			is_message ? M_PAGER_MESSAGE : 0, &info);
   }
@@ -762,7 +767,7 @@ int mutt_save_attachment (FILE *fp, BODY *m, char *path, int flags, HEADER *hdr)
 
 /* returns 0 on success, -1 on error */
 int mutt_decode_save_attachment (FILE *fp, BODY *m, char *path,
-					      int displaying, int flags)
+				 int displaying, int flags)
 {
   STATE s;
   unsigned int saved_encoding = 0;
@@ -770,10 +775,8 @@ int mutt_decode_save_attachment (FILE *fp, BODY *m, char *path,
   HEADER *saved_hdr = NULL;
 
   memset (&s, 0, sizeof (s));
-  s.flags = displaying ? M_DISPLAY : 0;
+  s.flags = (displaying ? M_DISPLAY : 0);
 
-  s.flags |= M_CHARCONV;
-  
   if (flags == M_SAVE_APPEND)
     s.fpout = fopen (path, "a");
   else if (flags == M_SAVE_OVERWRITE)
@@ -813,9 +816,16 @@ int mutt_decode_save_attachment (FILE *fp, BODY *m, char *path,
     saved_parts = m->parts;
     saved_hdr = m->hdr;
     mutt_parse_part (s.fpin, m);
+
+    /* display a readable version to the user */
+    if (m->noconv)
+      s.flags |= M_CHARCONV;
   }
   else
+  {
     s.fpin = fp;
+    s.flags |= M_CHARCONV;
+  }
 
   mutt_body_handler (m, &s);
 
