@@ -483,28 +483,6 @@ struct option_t MuttVars[] = {
   ** .pp
   ** Also see the ``$$record'' variable.
   */
-  { "forward_attachment", DT_BOOL, R_NONE, OPTFORWATTACH, 0 },
-  /*
-  ** .pp
-  ** When set, and you (tag-)forward attachments from the Attachment menu,
-  ** Mutt creates a new message with the specified attachments \fIonly\fP.
-  ** ``$$forward_format'' is used to set the default subject.
-  ** .pp
-  ** When unset, Mutt creates a pseudo-message comprising the tagged
-  ** attachments and forwards that. See the section on Forwarding a message
-  ** for details.
-  ** .pp
-  ** For example: if you have a message containing a image/gif,
-  ** application/octet-stream, or whatever attachment, you can go to the
-  ** Attachment menu and forward ONLY that attachment to someone else.
-  ** .pp
-  ** Basically, setting this variable produces the same effect as
-  ** explicitly saving the attachment(s), composing a new message and then
-  ** attaching the saved file(s).
-  */
-  { "forw_attachment",  DT_SYN, R_NONE, UL "forward_attachment", 0 },
-  /*
-  */
   { "forward_decode",	DT_BOOL, R_NONE, OPTFORWDECODE, 1 },
   /*
   ** .pp
@@ -662,6 +640,27 @@ struct option_t MuttVars[] = {
   */
   { "imap_home_namespace",	DT_STR, R_NONE, UL &ImapHomeNamespace, UL 0},
   /*
+  ** .pp
+  ** You normally want to see your personal folders alongside
+  ** your INBOX in the IMAP browser. If you see something else, you may set
+  ** this variable to the IMAP path to your folders.
+  */
+  { "imap_preconnect",	DT_STR, R_NONE, UL &ImapPreconnect, UL 0},
+  /*
+  ** .pp
+  ** If set, a shell command to be executed if mutt fails to establish
+  ** a connection to the server. This is useful for setting up secure
+  ** connections, e.g. with ssh(1). If the command returns a  nonzero
+  ** status, mutt gives up opening the server. Example:
+  ** .pp
+  ** imap_preconnect="ssh -f -q -L 1234:mailhost.net:143 mailhost.net
+  **                   sleep 20 < /dev/null > /dev/null"
+  ** .pp
+  ** Mailbox 'foo' on mailhost.net can now be reached as
+  ** '{localhost:1234}foo'.
+  ** .pp
+  ** NOTE: For this example to work, you must be able to log in to the
+  ** remote machine without having to enter a password.
   */
 #endif
   { "implicit_autoview", DT_BOOL,R_NONE, OPTIMPLICITAUTOVIEW, 0},
@@ -901,6 +900,14 @@ struct option_t MuttVars[] = {
   /*
   */
 
+  { "mime_forward_rest", DT_QUAD, R_NONE, OPT_MIMEFWDREST, M_YES },
+  /*
+  ** .pp
+  ** When forwarding multiple attachments of a MIME message from the recvattach
+  ** menu, attachments which cannot be decoded in a reasonable manner will
+  ** be attached to the newly composed message if this option is set.
+  */
+
 
 
   { "move",		DT_QUAD, R_NONE, OPT_MOVE, M_ASKNO },
