@@ -233,6 +233,7 @@ enum
   OPT_ABORT,
   OPT_RECALL,
   OPT_SUBJECT,
+  OPT_MIMEFWDREST,
   OPT_MAX
 };
 
@@ -272,7 +273,6 @@ enum
   OPTFCCATTACH,
   OPTFOLLOWUPTO,
   OPTFORCENAME,
-  OPTFORWATTACH,
   OPTFORWDECODE,
   OPTFORWQUOTE,
   OPTHDRS,
