@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1999 Thomas Roessler <roessler@guug.de>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -62,31 +63,29 @@
 
 #ifdef DL_STANDALONE
 
-static int invoke_dotlock(const char *path, int flags, int retry)
+static int invoke_dotlock (const char *path, int flags, int retry)
 {
   char cmd[LONG_STRING + _POSIX_PATH_MAX];
+  char f[SHORT_STRING + _POSIX_PATH_MAX];
   char r[SHORT_STRING];
-  char *f;
   
-  if(flags & DL_FL_RETRY)
-    snprintf(r, sizeof(r), "-r %d ", retry ? MAXLOCKATTEMPT : 0);
+  if (flags & DL_FL_RETRY)
+    snprintf (r, sizeof (r), "-r %d ", retry ? MAXLOCKATTEMPT : 0);
   
-  f = mutt_quote_filename(path);
+  mutt_quote_filename (f, sizeof (f), path);
   
-  snprintf(cmd, sizeof(cmd),
-	   "%s %s%s%s%s%s%s%s",
-	   DOTLOCK,
-	   flags & DL_FL_TRY ? "-t " : "",
-	   flags & DL_FL_UNLOCK ? "-u " : "",
-	   flags & DL_FL_USEPRIV ? "-p " : "",
-	   flags & DL_FL_FORCE ? "-f " : "",
-	   flags & DL_FL_UNLINK ? "-d " : "",
-	   flags & DL_FL_RETRY ? r : "",
-	   f);
+  snprintf (cmd, sizeof (cmd),
+	    "%s %s%s%s%s%s%s%s",
+	    NONULL (MuttDotlock),
+	    flags & DL_FL_TRY ? "-t " : "",
+	    flags & DL_FL_UNLOCK ? "-u " : "",
+	    flags & DL_FL_USEPRIV ? "-p " : "",
+	    flags & DL_FL_FORCE ? "-f " : "",
+	    flags & DL_FL_UNLINK ? "-d " : "",
+	    flags & DL_FL_RETRY ? r : "",
+	    f);
   
-  FREE(&f);
-
-  return mutt_system(cmd);
+  return mutt_system (cmd);
 }
 
 #else 
@@ -99,12 +98,11 @@ static int dotlock_file (const char *path, int retry)
 {
   int r;
   int flags = DL_FL_USEPRIV | DL_FL_RETRY;
-  
-  if(retry) retry = 1;
+
+  if (retry) retry = 1;
 
 retry_lock:
-  mutt_clear_error();
-  if((r = invoke_dotlock(path, flags, retry)) == DL_EX_EXIST)
+  if ((r = invoke_dotlock(path, flags, retry)) == DL_EX_EXIST)
   {
     if (!option (OPTNOCURSES))
     {
@@ -116,6 +114,7 @@ retry_lock:
       {
 	flags |= DL_FL_FORCE;
 	retry--;
+	mutt_clear_error ();
 	goto retry_lock;
       }
     } 
@@ -293,12 +292,12 @@ void mx_unlink_empty (const char *path)
 {
   int fd;
 #ifndef USE_DOTLOCK
-  char b;
+  struct stat sb;
 #endif
 
   if ((fd = open (path, O_RDWR)) == -1)
     return;
-  
+
   if (mx_lock_file (path, fd, 1, 0, 1) == -1)
   {
     close (fd);
@@ -308,11 +307,12 @@ void mx_unlink_empty (const char *path)
 #ifdef USE_DOTLOCK
   invoke_dotlock (path, DL_FL_UNLINK, 1);
 #else
-  if  (read (fd, &b, 1) != 1)
+  if (fstat (fd, &sb) == 0 && sb.st_size == 0)
     unlink (path);
 #endif
 
   mx_unlock_file (path, fd, 0);
+  close (fd);
 }
 
 /* try to figure out what type of mailbox ``path'' is
@@ -726,13 +726,14 @@ static int sync_mailbox (CONTEXT *ctx)
       
 #ifdef USE_IMAP
     case M_IMAP:
-      rc = imap_sync_mailbox (ctx);
+      /* extra argument means EXPUNGE */
+      rc = imap_sync_mailbox (ctx, 1);
       break;
 #endif /* USE_IMAP */
   }
 
 #if 0
-  if (!ctx->quiet && rc == -1)
+  if (!ctx->quiet && !ctx->shutup && rc == -1)
     mutt_error ( _("Could not synchronize mailbox %s!"), ctx->path);
 #endif
   
@@ -795,6 +796,9 @@ int mx_close_mailbox (CONTEXT *ctx)
     }
     mutt_expand_path (mbox, sizeof (mbox));
 
+#ifdef USE_IMAP
+    if (!mx_is_imap (ctx->path))
+#endif
     if (isSpool)
     {
       snprintf (buf, sizeof (buf), _("Move read messages to %s?"), mbox);
@@ -813,7 +817,7 @@ int mx_close_mailbox (CONTEXT *ctx)
   }
 
 #ifdef USE_IMAP
-  /* IMAP servers managed the OLD flag themselves */
+  /* IMAP servers manage the OLD flag themselves */
   if (ctx->magic != M_IMAP)
 #endif
   if (option (OPTMARKOLD))
@@ -858,30 +862,43 @@ int mx_close_mailbox (CONTEXT *ctx)
     return 0;
   }
   
-  if (!purge)
+#ifdef USE_IMAP
+  /* allow IMAP to preserve the deleted flag across sessions */
+  if (ctx->magic == M_IMAP)
   {
-    for (i = 0; i < ctx->msgcount; i++)
-      ctx->hdrs[i]->deleted = 0;
-    ctx->deleted = 0;
+    if (imap_sync_mailbox (ctx, purge) == -1)
+      return -1;
+    if (ctx->magic == M_IMAP && !purge)
+      mutt_message (_("%d kept."), ctx->msgcount);
   }
-
-  if (ctx->changed || ctx->deleted)
-  {
-    if (sync_mailbox (ctx) == -1)
-      return (-1);
-  }
-
-  if (move_messages)
-    mutt_message (_("%d kept, %d moved, %d deleted."),
-		  ctx->msgcount - ctx->deleted, read_msgs, ctx->deleted);
   else
-    mutt_message (_("%d kept, %d deleted."),
-		  ctx->msgcount - ctx->deleted, ctx->deleted);
+#endif
+  {
+    if (!purge)
+    {
+      for (i = 0; i < ctx->msgcount; i++)
+        ctx->hdrs[i]->deleted = 0;
+      ctx->deleted = 0;
+    }
 
-  if (ctx->msgcount == ctx->deleted &&
-      (ctx->magic == M_MMDF || ctx->magic == M_MBOX) &&
-      !mutt_is_spool(ctx->path) && !option (OPTSAVEEMPTY))
-    mx_unlink_empty (ctx->path);
+    if (ctx->changed || ctx->deleted)
+    {
+      if (sync_mailbox (ctx) == -1)
+        return -1;
+    }
+
+    if (move_messages)
+      mutt_message (_("%d kept, %d moved, %d deleted."),
+        ctx->msgcount - ctx->deleted, read_msgs, ctx->deleted);
+    else
+      mutt_message (_("%d kept, %d deleted."),
+        ctx->msgcount - ctx->deleted, ctx->deleted);
+
+    if (ctx->msgcount == ctx->deleted &&
+        (ctx->magic == M_MMDF || ctx->magic == M_MBOX) &&
+        !mutt_is_spool(ctx->path) && !option (OPTSAVEEMPTY))
+      mx_unlink_empty (ctx->path);
+  }
 
   mx_fastclose_mailbox (ctx);
 
@@ -975,6 +992,7 @@ void mx_update_tables(CONTEXT *ctx, int committing)
 int mx_sync_mailbox (CONTEXT *ctx)
 {
   int rc, i;
+  int purge = 1;
 
   if (ctx->dontwrite)
   {
@@ -1007,22 +1025,40 @@ int mx_sync_mailbox (CONTEXT *ctx)
     snprintf (buf, sizeof (buf), ctx->deleted == 1
 	     ? _("Purge %d deleted message?") : _("Purge %d deleted messages?"),
 	      ctx->deleted);
-    if ((rc = query_quadoption (OPT_DELETE, buf)) < 0)
+    if ((purge = query_quadoption (OPT_DELETE, buf)) < 0)
       return (-1);
-    else if (rc == M_NO)
+    else if (purge == M_NO)
     {
       if (!ctx->changed)
 	return 0; /* nothing to do! */
-      for (i = 0 ; i < ctx->msgcount ; i++)
-	ctx->hdrs[i]->deleted = 0;
-      ctx->deleted = 0;
+#ifdef USE_IMAP
+      /* let IMAP servers hold on to D flags */
+      if (ctx->magic != M_IMAP)
+#endif
+      {
+        for (i = 0 ; i < ctx->msgcount ; i++)
+          ctx->hdrs[i]->deleted = 0;
+        ctx->deleted = 0;
+      }
     }
   }
 
-  if ((rc = sync_mailbox (ctx)) == 0)
+#ifdef USE_IMAP
+  if (ctx->magic == M_IMAP)
+    rc = imap_sync_mailbox (ctx, purge);
+  else
+#endif
+  rc = sync_mailbox (ctx);
+  if (rc == 0)
   {
+#ifdef USE_IMAP
+    if (ctx->magic == M_IMAP && !purge)
+      mutt_message (_("%d kept."), ctx->msgcount);
+    else
+#endif
     mutt_message (_("%d kept, %d deleted."), ctx->msgcount - ctx->deleted,
-		  ctx->deleted);
+      ctx->deleted);
+
     sleep (1); /* allow the user time to read the message */
 
     if (ctx->msgcount == ctx->deleted &&
@@ -1034,8 +1070,12 @@ int mx_sync_mailbox (CONTEXT *ctx)
       return 0;
     }
 
-    mx_update_tables(ctx, 1);
-    mutt_sort_headers (ctx, 1); /* rethread from scratch */
+    /* if we haven't deleted any messages, we don't need to resort */
+    if (purge)
+    {
+      mx_update_tables(ctx, 1);
+      mutt_sort_headers (ctx, 1); /* rethread from scratch */
+    }
   }
 
   return (rc);
@@ -1136,15 +1176,39 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr, int flags)
 }
 
 /* check for new mail */
-int mx_check_mailbox (CONTEXT *ctx, int *index_hint)
+int mx_check_mailbox (CONTEXT *ctx, int *index_hint, int lock)
 {
+  int rc;
+
   if (ctx)
   {
+    if (ctx->locked) lock = 0;
+
     switch (ctx->magic)
     {
       case M_MBOX:
       case M_MMDF:
-	return (mbox_check_mailbox (ctx, index_hint));
+
+	if (lock)
+	{
+	  mutt_block_signals ();
+	  if (mbox_lock_mailbox (ctx, 0, 0) == -1)
+	  {
+	    mutt_unblock_signals ();
+	    return M_LOCKED;
+	  }
+	}
+	
+	rc = mbox_check_mailbox (ctx, index_hint);
+
+	if (lock)
+	{
+	  mutt_unblock_signals ();
+	  mbox_unlock_mailbox (ctx);
+	}
+	
+	return rc;
+
 
       case M_MH:
       case M_MAILDIR:
@@ -1327,16 +1391,6 @@ void mx_update_context (CONTEXT *ctx)
 
 
 
-  if (h->flagged)
-    ctx->flagged++;
-  if (h->deleted)
-    ctx->deleted++;
-  if (!h->read)
-  {
-    ctx->unread++;
-    if (!h->old)
-      ctx->new++;
-  }
   if (!ctx->pattern)
   {
     ctx->v2r[ctx->vcount] = ctx->msgcount;
@@ -1355,7 +1409,8 @@ void mx_update_context (CONTEXT *ctx)
     if (h2)
     {
       h2->superseded = 1;
-      mutt_score_message (h2);
+      if (option (OPTSCORE)) 
+	mutt_score_message (ctx, h2, 1);
     }
   }
 
@@ -1365,5 +1420,19 @@ void mx_update_context (CONTEXT *ctx)
   if (h->env->real_subj)
     hash_insert (ctx->subj_hash, h->env->real_subj, h, 1);
 
-  mutt_score_message (h);
+  if (option (OPTSCORE)) 
+    mutt_score_message (ctx, h, 0);
+  
+  if (h->changed)
+    ctx->changed = 1;
+  if (h->flagged)
+    ctx->flagged++;
+  if (h->deleted)
+    ctx->deleted++;
+  if (!h->read)
+  {
+    ctx->unread++;
+    if (!h->old)
+      ctx->new++;
+  }
 }
