@@ -45,36 +45,59 @@ static struct mapping_t PostponeHelp[] = {
 
 
 
-
 static short PostCount = 0;
-static time_t LastModify = 0;
 static CONTEXT *PostContext = NULL;
+static short UpdateNumPostponed = 0;
 
-int mutt_num_postponed (void)
+/* Return the number of postponed messages.
+ * if force is 0, use a cached value if it is costly to get a fresh
+ * count (IMAP) - else check.
+ */
+int mutt_num_postponed (int force)
 {
   struct stat st;
   CONTEXT ctx;
 
+  static time_t LastModify = 0;
+  static char *OldPostponed = NULL;
+
+  if (UpdateNumPostponed)
+  {
+    UpdateNumPostponed = 0;
+    force = 1;
+  }
+
+  if (Postponed != OldPostponed)
+  {
+    OldPostponed = Postponed;
+    LastModify = 0;
+    force = 1;
+  }
+
+  if (!Postponed)
+    return 0;
+
 #ifdef USE_IMAP
   /* LastModify is useless for IMAP */
-  if (Postponed && mx_is_imap (Postponed))
+  if (mx_is_imap (Postponed))
   {
-    PostCount = imap_mailbox_check (Postponed, 0);
-    PostCount = (PostCount < 0) ? 0 : PostCount;
-    dprint(2, (debugfile,
-      "mutt_num_postponed: %d postponed IMAP messages found.\n", PostCount));
-
+    if (force)
+    {
+      PostCount = imap_mailbox_check (Postponed, 0);
+      PostCount = (PostCount < 0) ? 0 : PostCount;
+      dprint(2, (debugfile,
+        "mutt_num_postponed: %d postponed IMAP messages found.\n", PostCount));
+    }
     return PostCount;
   }
 #endif
 
-  if (!Postponed || stat (Postponed, &st) == -1)
+  if (stat (Postponed, &st) == -1)
   {
      PostCount = 0;
      LastModify = 0;
      return (0);
-  } 
-
+  }
 
   if (S_ISDIR (st.st_mode))
   {
@@ -107,6 +130,11 @@ int mutt_num_postponed (void)
   return (PostCount);
 }
 
+void mutt_update_num_postponed (void)
+{
+  UpdateNumPostponed = 1;
+}
+
 static void post_entry (char *s, size_t slen, MUTTMENU *menu, int entry)
 {
   CONTEXT *ctx = (CONTEXT *) menu->data;
@@ -251,7 +279,7 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
     return (-1);
   }
 
-  if (mutt_prepare_template (PostContext, hdr, h, 0) < 0)
+  if (mutt_prepare_template (NULL, PostContext, hdr, h, 0) < 0)
   {
     mx_fastclose_mailbox (PostContext);
 #ifdef USE_IMAP
@@ -353,23 +381,31 @@ static void free_body_list (BODY **bp)
 }
 
 
-int mutt_prepare_template (CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
+int mutt_prepare_template (FILE *fp, CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
 			       short weed)
 {
   PARAMETER *par, **ppar;
-  MESSAGE *msg;
+  MESSAGE *msg = NULL;
   char file[_POSIX_PATH_MAX];
   BODY *b;
   LIST *p, **q;
 
   FILE *bfp;
-  
-  if ((msg = mx_open_message (ctx, hdr->msgno)) == NULL)
+
+  if (!fp && (msg = mx_open_message (ctx, hdr->msgno)) == NULL)
     return (-1);
 
+  if (!fp) fp = msg->fp;
+
   /* parse the message header */
-  fseek (msg->fp, hdr->offset, 0);
-  newhdr->env = mutt_read_rfc822_header (msg->fp, newhdr, 1, weed);
+  fseek (fp, hdr->offset, 0);
+  newhdr->env = mutt_read_rfc822_header (fp, newhdr, 1, weed);
+
+  /* NOTE: We do have easy access to the entire MIME structure of
+   * the message to be recalled here.  We could use this to 
+   * considerably simplify the code below.
+   */
+
   mutt_free_body (&newhdr->content);
   
   /* weed user-agent, x-mailer - we don't want them here */
@@ -394,7 +430,7 @@ int mutt_prepare_template (CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
   safe_free ((void **) &newhdr->env->mail_followup_to);
 
   /* make sure we parse the message */
-  mutt_parse_part (msg->fp, hdr->content);
+  mutt_parse_part (fp, hdr->content);
 
   {
     /* copy the MIME structure */
@@ -417,7 +453,7 @@ int mutt_prepare_template (CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
       (*ppar)->value = safe_strdup (par->value);
     }
     
-    bfp = msg->fp;
+    bfp = fp;
   }
 
   
@@ -453,8 +489,8 @@ int mutt_prepare_template (CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
     {
       mutt_free_envelope (&newhdr->env);
       free_body_list (&newhdr->content);
-      if (bfp != msg->fp) fclose (bfp);
-      mx_close_message (&msg);
+      if (bfp != fp) fclose (bfp);
+      if (msg) mx_close_message (&msg);
       return -1;
     }
     
@@ -472,8 +508,8 @@ int mutt_prepare_template (CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
   }
 
   /* that's it. */
-  if (bfp != msg->fp) fclose (bfp);
-  mx_close_message (&msg);
+  if (bfp != fp) fclose (bfp);
+  if (msg) mx_close_message (&msg);
 
   return 0;
 }
